@@ -28,6 +28,7 @@ use std::{
     ops::Deref,
 };
 
+#[allow(clippy::derive_partial_eq_without_eq)]
 mod gen {
     include!("arrow.flight.protocol.rs");
 }
@@ -67,6 +68,9 @@ pub use gen::Ticket;
 
 pub mod utils;
 
+#[cfg(feature = "flight-sql-experimental")]
+pub mod sql;
+
 use flight_descriptor::DescriptorType;
 
 /// SchemaAsIpc represents a pairing of a `Schema` with IpcWriteOptions
