@@ -17,28 +17,20 @@
 
 extern crate arrow;
 
-#[cfg(feature = "csv")]
 use arrow::csv;
-#[cfg(feature = "prettyprint")]
 use arrow::util::pretty::print_batches;
 use std::fs::File;
 
 fn main() {
-    #[cfg(feature = "csv")]
-    {
-        let path = format!(
-            "{}/test/data/uk_cities_with_headers.csv",
-            env!("CARGO_MANIFEST_DIR")
-        );
-        let file = File::open(path).unwrap();
-        let builder = csv::ReaderBuilder::new()
-            .has_header(true)
-            .infer_schema(Some(100));
-        let mut csv = builder.build(file).unwrap();
-        let _batch = csv.next().unwrap().unwrap();
-        #[cfg(feature = "prettyprint")]
-        {
-            print_batches(&[_batch]).unwrap();
-        }
-    }
+    let path = format!(
+        "{}/test/data/uk_cities_with_headers.csv",
+        env!("CARGO_MANIFEST_DIR")
+    );
+    let file = File::open(path).unwrap();
+    let builder = csv::ReaderBuilder::new()
+        .has_header(true)
+        .infer_schema(Some(100));
+    let mut csv = builder.build(file).unwrap();
+    let batch = csv.next().unwrap().unwrap();
+    print_batches(&[batch]).unwrap();
 }
