@@ -1229,8 +1229,8 @@ mod tests {
 
     #[test]
     fn test_primitive_array_add_dyn_dict() {
-        let key_builder = PrimitiveBuilder::<Int8Type>::new(3);
-        let value_builder = PrimitiveBuilder::<Int32Type>::new(2);
+        let key_builder = PrimitiveBuilder::<Int8Type>::with_capacity(3);
+        let value_builder = PrimitiveBuilder::<Int32Type>::with_capacity(2);
         let mut builder = PrimitiveDictionaryBuilder::new(key_builder, value_builder);
         builder.append(5).unwrap();
         builder.append(6).unwrap();
@@ -1239,8 +1239,8 @@ mod tests {
         builder.append(9).unwrap();
         let a = builder.finish();
 
-        let key_builder = PrimitiveBuilder::<Int8Type>::new(3);
-        let value_builder = PrimitiveBuilder::<Int32Type>::new(2);
+        let key_builder = PrimitiveBuilder::<Int8Type>::with_capacity(3);
+        let value_builder = PrimitiveBuilder::<Int32Type>::with_capacity(2);
         let mut builder = PrimitiveDictionaryBuilder::new(key_builder, value_builder);
         builder.append(6).unwrap();
         builder.append(7).unwrap();
@@ -1270,8 +1270,8 @@ mod tests {
         assert!(c.is_null(3));
         assert_eq!(10, c.value(4));
 
-        let key_builder = PrimitiveBuilder::<Int8Type>::new(3);
-        let value_builder = PrimitiveBuilder::<Int32Type>::new(2);
+        let key_builder = PrimitiveBuilder::<Int8Type>::with_capacity(3);
+        let value_builder = PrimitiveBuilder::<Int32Type>::with_capacity(2);
         let mut builder = PrimitiveDictionaryBuilder::new(key_builder, value_builder);
         builder.append(5).unwrap();
         builder.append_null();
@@ -1313,8 +1313,8 @@ mod tests {
 
     #[test]
     fn test_primitive_array_subtract_dyn_dict() {
-        let key_builder = PrimitiveBuilder::<Int8Type>::new(3);
-        let value_builder = PrimitiveBuilder::<Int32Type>::new(2);
+        let key_builder = PrimitiveBuilder::<Int8Type>::with_capacity(3);
+        let value_builder = PrimitiveBuilder::<Int32Type>::with_capacity(2);
         let mut builder = PrimitiveDictionaryBuilder::new(key_builder, value_builder);
         builder.append(15).unwrap();
         builder.append(8).unwrap();
@@ -1323,8 +1323,8 @@ mod tests {
         builder.append(20).unwrap();
         let a = builder.finish();
 
-        let key_builder = PrimitiveBuilder::<Int8Type>::new(3);
-        let value_builder = PrimitiveBuilder::<Int32Type>::new(2);
+        let key_builder = PrimitiveBuilder::<Int8Type>::with_capacity(3);
+        let value_builder = PrimitiveBuilder::<Int32Type>::with_capacity(2);
         let mut builder = PrimitiveDictionaryBuilder::new(key_builder, value_builder);
         builder.append(6).unwrap();
         builder.append(7).unwrap();
@@ -1354,8 +1354,8 @@ mod tests {
         assert!(c.is_null(3));
         assert_eq!(8, c.value(4));
 
-        let key_builder = PrimitiveBuilder::<Int8Type>::new(3);
-        let value_builder = PrimitiveBuilder::<Int32Type>::new(2);
+        let key_builder = PrimitiveBuilder::<Int8Type>::with_capacity(3);
+        let value_builder = PrimitiveBuilder::<Int32Type>::with_capacity(2);
         let mut builder = PrimitiveDictionaryBuilder::new(key_builder, value_builder);
         builder.append(5).unwrap();
         builder.append_null();
@@ -1397,8 +1397,8 @@ mod tests {
 
     #[test]
     fn test_primitive_array_multiply_dyn_dict() {
-        let key_builder = PrimitiveBuilder::<Int8Type>::new(3);
-        let value_builder = PrimitiveBuilder::<Int32Type>::new(2);
+        let key_builder = PrimitiveBuilder::<Int8Type>::with_capacity(3);
+        let value_builder = PrimitiveBuilder::<Int32Type>::with_capacity(2);
         let mut builder = PrimitiveDictionaryBuilder::new(key_builder, value_builder);
         builder.append(5).unwrap();
         builder.append(6).unwrap();
@@ -1407,8 +1407,8 @@ mod tests {
         builder.append(9).unwrap();
         let a = builder.finish();
 
-        let key_builder = PrimitiveBuilder::<Int8Type>::new(3);
-        let value_builder = PrimitiveBuilder::<Int32Type>::new(2);
+        let key_builder = PrimitiveBuilder::<Int8Type>::with_capacity(3);
+        let value_builder = PrimitiveBuilder::<Int32Type>::with_capacity(2);
         let mut builder = PrimitiveDictionaryBuilder::new(key_builder, value_builder);
         builder.append(6).unwrap();
         builder.append(7).unwrap();
@@ -1441,8 +1441,8 @@ mod tests {
 
     #[test]
     fn test_primitive_array_divide_dyn_dict() {
-        let key_builder = PrimitiveBuilder::<Int8Type>::new(3);
-        let value_builder = PrimitiveBuilder::<Int32Type>::new(2);
+        let key_builder = PrimitiveBuilder::<Int8Type>::with_capacity(3);
+        let value_builder = PrimitiveBuilder::<Int32Type>::with_capacity(2);
         let mut builder = PrimitiveDictionaryBuilder::new(key_builder, value_builder);
         builder.append(15).unwrap();
         builder.append(6).unwrap();
@@ -1451,8 +1451,8 @@ mod tests {
         builder.append(9).unwrap();
         let a = builder.finish();
 
-        let key_builder = PrimitiveBuilder::<Int8Type>::new(3);
-        let value_builder = PrimitiveBuilder::<Int32Type>::new(2);
+        let key_builder = PrimitiveBuilder::<Int8Type>::with_capacity(3);
+        let value_builder = PrimitiveBuilder::<Int32Type>::with_capacity(2);
         let mut builder = PrimitiveDictionaryBuilder::new(key_builder, value_builder);
         builder.append(5).unwrap();
         builder.append(3).unwrap();
@@ -1482,8 +1482,8 @@ mod tests {
         assert!(c.is_null(3));
         assert_eq!(18, c.value(4));
 
-        let key_builder = PrimitiveBuilder::<Int8Type>::new(3);
-        let value_builder = PrimitiveBuilder::<Int32Type>::new(2);
+        let key_builder = PrimitiveBuilder::<Int8Type>::with_capacity(3);
+        let value_builder = PrimitiveBuilder::<Int32Type>::with_capacity(2);
         let mut builder = PrimitiveDictionaryBuilder::new(key_builder, value_builder);
         builder.append(5).unwrap();
         builder.append_null();
@@ -1668,8 +1668,8 @@ mod tests {
         assert!(c.is_null(3));
         assert_eq!(4, c.value(4));
 
-        let key_builder = PrimitiveBuilder::<Int8Type>::new(3);
-        let value_builder = PrimitiveBuilder::<Int32Type>::new(2);
+        let key_builder = PrimitiveBuilder::<Int8Type>::with_capacity(3);
+        let value_builder = PrimitiveBuilder::<Int32Type>::with_capacity(2);
         let mut builder = PrimitiveDictionaryBuilder::new(key_builder, value_builder);
         builder.append(5).unwrap();
         builder.append_null();
@@ -1936,14 +1936,14 @@ mod tests {
     #[test]
     #[should_panic(expected = "DivideByZero")]
     fn test_primitive_array_divide_dyn_by_zero_dict() {
-        let key_builder = PrimitiveBuilder::<Int8Type>::new(1);
-        let value_builder = PrimitiveBuilder::<Int32Type>::new(1);
+        let key_builder = PrimitiveBuilder::<Int8Type>::with_capacity(1);
+        let value_builder = PrimitiveBuilder::<Int32Type>::with_capacity(1);
         let mut builder = PrimitiveDictionaryBuilder::new(key_builder, value_builder);
         builder.append(15).unwrap();
         let a = builder.finish();
 
-        let key_builder = PrimitiveBuilder::<Int8Type>::new(1);
-        let value_builder = PrimitiveBuilder::<Int32Type>::new(1);
+        let key_builder = PrimitiveBuilder::<Int8Type>::with_capacity(1);
+        let value_builder = PrimitiveBuilder::<Int32Type>::with_capacity(1);
         let mut builder = PrimitiveDictionaryBuilder::new(key_builder, value_builder);
         builder.append(0).unwrap();
         let b = builder.finish();
